#ifndef __MESHOBJ_H__
#define __MESHOBJ_H__

#include "mmanager.h"
#include "spointer.h"
#include "textureobj.h"
#include "drawobj.h"
#include <d3dx9.h>

class CMMMesh : public CMMDrawObject
{
protected:
	LPD3DXMESH Mesh;
	D3DMATERIAL9* MeshMaterials; // TODO: Da li treba dodati objekat za materijale???
	CMMPointer<CMMTexture>* MeshTextures;
	DWORD dwNumMaterials;
	void CleanUp()
	{
		SAFE_DELETE_ARRAY( MeshMaterials );
		if( MeshTextures )
		{
			for( DWORD i = 0; i < dwNumMaterials; i++ )
			{
				MeshTextures[i] = NULL;
			}
		SAFE_DELETE_ARRAY( MeshTextures );
		}
		SAFE_RELEASE( Mesh );
	};

public:
	CMMMesh( LPSTR name, LPSTR type = "MESH" ) : CMMDrawObject ( name, type )
	{
		Mesh = NULL;
		MeshMaterials = NULL;
		MeshTextures = NULL;
		dwNumMaterials = NULL;
	};

	~CMMMesh()
	{
		CleanUp();
	};

	HRESULT LoadMesh( LPCTSTR pFilename, DWORD Options, LPDIRECT3DDEVICE9 pD3DDevice )
	{
		CleanUp();

		LPD3DXBUFFER pD3DXMtrlBuffer;

		D3DXLoadMeshFromX( pFilename, Options, pD3DDevice, NULL,
			&pD3DXMtrlBuffer, NULL, &dwNumMaterials, &Mesh);
		
		D3DXMATERIAL* d3dxMaterials = (D3DXMATERIAL*)pD3DXMtrlBuffer->GetBufferPointer();
		MeshMaterials = new D3DMATERIAL9[dwNumMaterials];
		MeshTextures  = new CMMPointer<CMMTexture>[dwNumMaterials];


		for( DWORD i=0; i<dwNumMaterials; i++ )
		{
			MeshMaterials[i] = d3dxMaterials[i].MatD3D;
			MeshMaterials[i].Ambient = MeshMaterials[i].Diffuse;
			MeshTextures[i] = NULL;
			if( d3dxMaterials[i].pTextureFilename != NULL &&
				lstrlen(d3dxMaterials[i].pTextureFilename) > 0 )
			{
				if( !CMMObject::MemoryManager->FindObject<CMMTexture>( d3dxMaterials[i].pTextureFilename,
					"TEX", (CMMTexture*)&MeshTextures[i], TRUE ) )
				{
					MeshTextures[i]->LoadTexture( pD3DDevice, d3dxMaterials[i].pTextureFilename);
				};
			};
        };
		pD3DXMtrlBuffer->Release();
	};

	void Draw( LPDIRECT3DDEVICE9 pD3DDevice )
	{
		for( DWORD i=0; i<dwNumMaterials; i++ )
		{
			pD3DDevice->SetMaterial( &MeshMaterials[i] );
			pD3DDevice->SetTexture( 0, *(CMMTexture*)&MeshTextures[i] );
			Mesh->DrawSubset( i );
		};
	};
};

#endif //#ifndef __MESHOBJ_H__