#ifndef __MMANAGER_H__
#define __MMANAGER_H__

#include <list>
#include <windows.h>
#include <assert.h>

class CMMObject;




//------------------------------------------------------------
// CMManager klasa - sluzi za rad sa objektima u memoriji
//------------------------------------------------------------
class CMManager
{
friend CMMObject;

protected:
	std::list<CMMObject*> Objects;

public:
	std::list<CMMObject*>::iterator AddObject( CMMObject* obj );
	void RemoveObject( std::list<CMMObject*>::iterator it );
	template <typename T>
	BOOL FindObject( LPCSTR name, LPCSTR type, T* obj, BOOL createnew );
};




//------------------------------------------------------------
// CMMObject klasa - svi objekti nasledjuju ovu klasu
//------------------------------------------------------------
class CMMObject
{
friend CMManager;

protected:
	long refCount;
	LPSTR Name;
	LPSTR Type;
	std::list<CMMObject*>::iterator It;
	HRESULT AllocateStr( LPCSTR name, LPSTR *newname );

public:
	static CMManager* MemoryManager;
	CMMObject( LPSTR name, LPSTR type );
	virtual ~CMMObject();
	void AddRef();
	void Release();
};

#define SAFE_DELETE(p)       { if(p) { delete (p);     (p)=NULL; } }
#define SAFE_DELETE_ARRAY(p) { if(p) { delete[] (p);   (p)=NULL; } }
#define SAFE_RELEASE(p)      { if(p) { (p)->Release(); (p)=NULL; } }

#endif //#ifndef __MMANAGER_H__